#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"

#ifndef MASKTABLE
#define MASKTABLE 1
unsigned char masktable[8] = {0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01};
#endif

static 
int get_int ( FILE *fp )
{
	int i;
	int c;

	c = fgetc( fp );
	for( ;; ){
		if ( c == '#' ){
			while( c != '\n' ) {
				c = fgetc( fp );
				if ( c==EOF ) return 0;
			}
		}
		if ( c >= '0' && c <= '9' ) break; 
		c = fgetc( fp );
	}
	i = 0;
	while (1) {
		i = (i*10) + ((unsigned char)c - '0');
		c = fgetc(fp);
		if (c==EOF) return 0;
		if (c<'0' || c>'9') break;
	}
	return i;
}

/**
	Function to read a PNM image
	P4 - pbm (portable bit  map) raw format 
	P5 - pgm (portable gray map) raw format 
	P6 - ppm (portable pix  map) raw format (Nov, 2004. by Abigail Martínez)
**/
MATRIX * read_pnm( char *path )
{
	FILE *fp;
	MATRIX *IMG;
	char c1, c2;
	int type, rows, cols, greytones, colors, n, i, j;
	int element_size;
	unsigned char *paux, *buf;
	RGB *prgb;

	if ( (fp=fopen( path, "r"))==NULL ) {
		fprintf ( stderr, "ERROR: Can't open output file: %s\n", path );
		return NULL; 
	}

	c1 = fgetc( fp );
	c2 = fgetc( fp );
	if ( c1 != 'P' && ( c2!='4' || c2!='5' || c2!='6' ) ) {
		fprintf ( stderr, "ERROR: Format no supported\n" );
		return NULL; 
	}
	if( c2 == '4' )	type = 1;
	else		type = ( c2=='5' ) ? 5 : 6;

	cols = get_int( fp );
	if ( cols <= 0 ) return NULL;
	rows = get_int( fp );
	if ( rows <= 0 ) return NULL;

	element_size = 1;
	if ( type == 5 ) greytones = get_int( fp );
	else if ( type == 6 ) { 
		colors = get_int( fp );
		element_size = sizeof(RGB);
	}

	IMG = matrix_allocate( rows, cols, element_size );	
	if ( !IMG ) {
		fprintf ( stderr, "ERROR: No memory\n" );
		return NULL; 
	}
	IMG->rows = rows;
	IMG->cols = cols;
	IMG->type = type;
	IMG->element_size = element_size;

//	fprintf ( stderr, "type %d\n", type );

	if (type == 6) {	// RGB
		for( i=0; i<rows; i++ ) {
			prgb = (RGB *)IMG->ptr[i];
			if ( fread( prgb, element_size, cols, fp ) != cols ){
				fprintf ( stderr, "ERROR: reading file: %s\n", path );
				matrix_free( IMG );	
				return NULL; 
			}
			//printf("R:%d G:%d B:%d \n",prgb[50].R, prgb[50].G, prgb[50].B);
		}
	}

	else if ( type == 5 ) {	// Greytones
		for( i=0; i<rows; i++ ) {
			paux = (unsigned char*)IMG->ptr[i];
			if ( fread( paux, 1, cols, fp ) != cols ){
				fprintf ( stderr, "ERROR: reading file: %s\n", path );
				matrix_free( IMG );	
				return NULL; 
			}
		}
	}

	else {			// Binary
		n = pixels2bytes( cols ); 
		buf = (unsigned char *)malloc( n );

		for( i=0; i<rows; i++ ){
			if ( fread ( buf, 1, n, fp) != n ) {
				fprintf ( stderr, "ERROR: reading file: %s\n", path );
				matrix_free( IMG );	
				return NULL; 
			}
			paux = (unsigned char *) IMG->ptr[i];
			for ( j=0; j<cols; j++ ) {
				if ( buf[ j>>3 ] & masktable[ j & 0x0007 ] )
					*paux++ = 0x00;
				else *paux++ = 0xff;
			}
		}
		free( buf );
	}
		
	fclose ( fp );
	return IMG;
}

/**
	An easy function to write
	PNM images
**/
int write_pnm( MATRIX *IN, char *path )
{
	FILE *fp;
	int i, n, j;
	unsigned char *paux, *buf;	

	/** P4 - pbm (portable bit map) raw format 
		P5 - pgm (portable bir map) raw format **/

	if ( (fp=fopen( path, "w"))==NULL ) {
		fprintf ( stderr, "ERROR: Can't open output file: %s\n", path );
		return 1; 
	}

	if ( IN->type==1 || IN->type==5 ) {
		if ( IN->type==1) 
			fprintf( fp, "P4\n" );
		else {
			fprintf( fp, "P5\n" );
			fprintf( fp, "%d %d\n", IN->cols, IN->rows );
		}	
		fprintf( fp, "# scimagen software. CINVESTAV\n" );
		fprintf( fp, "#	 computer science section\n" );

		
		if ( IN->type==5) { 
			fprintf( fp, "255\n" );

			for( i=0; i<IN->rows; i++ ){
				paux = (unsigned char *) IN->ptr[i];
				if ( fwrite ( paux, 1, IN->cols, fp) != IN->cols ) {
					fprintf ( stderr, "ERROR: writing file: %s\n", path );
					return 1; 
				}
			}
		}
		else {
			fprintf( fp, "%d %d\n", IN->cols, IN->rows );
			n = pixels2bytes( IN->cols ); 
			buf = (unsigned char *)malloc( n );

			for( i=0; i<IN->rows; i++ ){
				paux = (unsigned char *) IN->ptr[i];
				memset( buf, 0xff, n );
				for ( j=0; j< IN->cols; j++ ) {
					if ( *paux++ > 0 )
						buf[ j>>3 ] &= ~masktable[ j & 0x0007 ];
				}

				if ( fwrite ( buf, 1, n, fp) != n ) {
					fprintf ( stderr, "ERROR: writing file: %s\n", path );
					return 1; 
				}
			}
			free( buf );
		}

		fclose ( fp );
	}
	else {
		fprintf ( stderr, "ERROR: Ouput format no supported\n" );
		return 1; 
	}

	return 0; /** OK! **/
}

	
